class PypBoy(object):
    # word_list = ["hello", "shell", "bells", "hells"]

    def __init__(self, word_list):
        self.word_list = word_list

        self.possibles = []
        # self.example_list = [
        #     "LOYALIST",
        #     "FEVERISH",
        #     "CITIZENS",
        #     "RESULTED",
        #     "SUDDENLY",
        #     "UNWANTED",
        #     "HUMANITY",
        #     "CONQUEST",
        #
        # ]

    def compare(self, guess_str, word, num_correct):
        correct = 0
        for x, y in zip(guess_str, word):
            print(f"Comparing {x} amd {y}")
            if x == y:
                print("Match!")
                correct += 1

        print(correct)

        if int(correct) == int(num_correct):
            correct = 0
            self.possibles.append(word)
            print(self.possibles)
            return True

    def guess(self, guess_str, num_correct):

        new_list = []
        print(guess_str)
        print(num_correct)

        for word in self.word_list:
            if word == guess_str:
                pass
            else:
                if self.compare(guess_str, word, num_correct):
                    self.possibles.append(word)

        self.word_list = self.possibles

        return self.word_list

    class Stats(object):
        def __init__(self):
            self.ledger = {}

        def add(self, guessed_word, ):
            entry = {}

    class CLI(object):
        def __init__(self):
            pass


import PySimpleGUI as Gui

from pyp_boy.common.tools import prepare_wordlist
from pyp_boy.gui.hacking import HackingWindow

GUI = Gui

WORD_MAX = 20


class PypBoyGUI(object):
    class MainWindowLayout(object):
        def __init__(self):
            self.header_col = [
                [GUI.Text("PypBoy!", justification="center", )]
            ]

            self.sep_col_1 = [
                [GUI.VSep()]
            ]

            self.sep_col_2 = [
                [GUI.VSep()]
            ]

            self.header_frame = [
                [GUI.Column(layout=self.sep_col_1, justification="left"),
                 GUI.Column(layout=self.header_col, justification="center", size=(200, 20)),
                 GUI.Column(layout=self.sep_col_2, justification="right")]
            ]

            self.word_frame = [
                [GUI.Text("Here's where you'll enter the words on the terminal")],
                [GUI.HSeparator()],
                [GUI.Text("Press 'Add Word' below to get started!")]
            ]

            self.button_frame_left = [
                [GUI.Button("Add Word", key='ADD_WORD_BUTTON')],
                [GUI.Button("Remove Word", key="REMOVE_WORD_BUTTON", visible=False)],

            ]

            self.button_frame_center = [
                [GUI.Button("Lock In!", enable_events=True, key='LOCK_IN_BUTTON', tooltip="Start hacking!",
                            visible=False)],
                [GUI.Button("Clear All", enable_events=True, key='CLEAR_ALL_BUTTON', tooltip="Clear all words",
                            visible=False)],
                [GUI.Button("Output Values to Console", key="VALS_OUT_BUTTON", visible=False)]
            ]

            self.button_frame_right = [
                [GUI.Button("Quit", enable_events=True, key='QUIT_BUTTON')],
            ]

    class MainWindow(object):

        def __init__(self, title):
            self.input_count = 0
            self.invisible_num = 0
            GUI.theme("DarkGreen2")
            self.Layout = PypBoyGUI.MainWindowLayout()

            header_frame = self.Layout.header_frame
            word_frame = self.Layout.word_frame
            bf_left = self.Layout.button_frame_left
            bf_center = self.Layout.button_frame_center
            bf_right = self.Layout.button_frame_right

            layout = [
                [GUI.Frame("", layout=header_frame, )],
                [GUI.VSep(), GUI.Frame('', layout=word_frame, key='WORD_FRAME'), GUI.VSep()],
                [GUI.Frame("", layout=bf_left, key='BF_LEFT'), GUI.Frame("", layout=bf_center, key='BF_CENTER'),
                 GUI.Frame("", layout=bf_right, key='BF_RIGHT')]

            ]

            self.win = GUI.Window(title, layout=layout, element_justification="center")
            self.last_removed = False

        def add_word_box(self):
            if self.invisible_num >= 1:
                self.win[f'WORD_{self.input_count}'].update('', visible=True)
                self.invisible_num -= 1
            else:
                self.win.extend_layout(self.win['WORD_FRAME'], [[GUI.Input(key=f"WORD_{self.input_count}")]])

            self.win['REMOVE_WORD_BUTTON'].update(visible=True)
            self.win['VALS_OUT_BUTTON'].update(visible=True)
            if self.input_count >= 2:
                self.win['LOCK_IN_BUTTON'].update(visible=True)
                self.win['CLEAR_ALL_BUTTON'].update(visible=True)

        def run_window(self):
            while True:
                event, vals = self.win.read(timeout=100)

                if event is None:
                    self.win.close()
                    exit()

                if event == "QUIT_BUTTON":
                    self.win.close()
                    exit()

                if event == 'ADD_WORD_BUTTON':
                    print(self.input_count)

                    if self.input_count >= WORD_MAX:
                        GUI.popup_error("You have the maximum number of words!")
                        continue

                    if self.input_count >= 1:
                        if vals[f'WORD_{self.input_count}'].replace(' ', '') == '':
                            GUI.popup_error(f"Please put a word in slot {self.input_count} before trying to add "
                                            f"another word")
                            continue

                    self.input_count += 1
                    self.add_word_box()

                    #     self.win.refresh()
                    #     continue
                    #
                    # try:
                    #     if vals[f'WORD_{self.input_count}'].replace(' ', '') == '':

                    #         continue
                    #
                    # except KeyError:
                    #     self.input_count += 1
                    #     self.win.extend_layout(self.win['WORD_FRAME'], [[GUI.Input(key=f"WORD_{self.input_count}")]])
                    #     continue

                    #
                    # if self.input_count >= 1:
                    #     self.win["REMOVE_WORD_BUTTON"].update(visible=True)
                    #
                    # self.last_removed = False
                    # self.win['REMOVE_WORD_BUTTON'].update(visible=True)
                    # self.win.refresh()
                    # self.input_count += 1

                if event == "REMOVE_WORD_BUTTON":
                    self.win[f'WORD_{self.input_count}'].update('', visible=False)
                    self.input_count -= 1
                    if self.input_count == 0:
                        self.win['REMOVE_WORD_BUTTON'].update(visible=False)
                        self.win['VALS_OUT_BUTTON'].update(visible=False)
                    self.invisible_num += 1

                    if self.input_count < 2:
                        self.win['LOCK_IN_BUTTON'].update(visible=False)
                        self.win['CLEAR_ALL_BUTTON'].update(visible=False)

                if event == "VALS_OUT_BUTTON":
                    print(vals)
                    print(dir(vals))
                    print(dir(event))

                if event == "LOCK_IN_BUTTON":
                    word_list = []
                    for w in vals.items():
                        word_list.append(w[1])
                    wl = prepare_wordlist(word_list)
                    print(wl)
                    self.win.close()
                    hacking_window = HackingWindow(wl)



import PySimpleGUI as gui

import pyp_boy


class HackingWindow(pyp_boy.PypBoy):

    def _add_buttons(self):
        acc = 0

        while True:

            if acc >= 3:
                break

            for word in self.word_cols[acc]:
                self.window.extend_layout(self.window[f'WCOL_{str(acc + 1)}'],
                                          [[gui.Button(word, key=f"WBUTTON_{word}")]])

            acc += 1

    def _col_sort(self):
        col1 = []
        col2 = []
        col3 = []

        acc = 0

        for word in self.og_wordlist:
            acc += 1
            if acc == 1:
                col1.append(word.upper())
            elif acc == 2:
                col2.append(word.upper())
            elif acc == 3:
                col3.append(word.upper())
                acc = 0

        return col1, col2, col3

    def __init__(self, wordlist):
        super(HackingWindow, self).__init__(wordlist)
        self.og_wordlist = wordlist
        self.word_cols = self._col_sort()
        self.wl = self.og_wordlist
        self.hidden_buttons = []
        self.active_guess = None

        acc = 0

        self.col_1 = []
        self.col_2 = []
        self.col_3 = []

        self.word_frame = [
            [
                gui.Text("Select a word in the terminal you're hacking and then tell PypBoy the number you have "
                         "correct and the word your chose.", key="INSTRUCTION_TEXT")
            ],
            [
                gui.Column(self.col_1, key="WCOL_1"),
                gui.Column(self.col_2, key="WCOL_2"),
                gui.Column(self.col_3, key="WCOL_3")
            ]
        ]

        self.guess_frame = [
            [
                gui.Text("Guessed word:"),
                gui.VSep(),
                gui.Text("", key="GUESSED_WORD", size=(10, 1))
            ],
            [
                gui.Text("Correct:"),
                gui.VSep(),
                gui.Spin(
                    list(range(1, len(wordlist[0]) + 1)),
                    key="NUM_CORRECT"
                )
            ]
        ]

        self.button_frame = [
            [
                gui.Button("Quit", key="HW_QUIT_BUTTON"),
                gui.Button("Guess", key="GUESS_BUTTON", visible=False),
                gui.Button("Undo", key="UNDO_BUTTON", visible=False, )
            ]
        ]

        self.layout = [
            [gui.Frame('', layout=self.word_frame)],
            [gui.Frame('', layout=self.guess_frame, visible=False, key="GUESS_FRAME",
                       element_justification='center')],
            [gui.Frame('', layout=self.button_frame)]
        ]

        self.window = gui.Window("PypBoy (TM)", layout=self.layout, finalize=True)
        self._add_buttons()

        while True:
            event, vals = self.window.read(timeout=100)

            if event is None or event == "HW_QUIT_BUTTON":
                self.window.close()
                break

            if "WBUTTON_" in event:
                g_word = str(event).split("_")[-1]
                self.active_guess = g_word

                self.window["GUESS_FRAME"].update(visible=True)
                self.window["GUESSED_WORD"].update(g_word)
                self.window['UNDO_BUTTON'].update(visible=True)
                self.window['GUESS_BUTTON'].update(visible=True)

                for w in self.wl:
                    button_tag = f"WBUTTON_{w}"
                    if not w == g_word:

                        self.window[button_tag].update(visible=False)
                        self.hidden_buttons.append(button_tag)
                    else:
                        self.window[button_tag].update(disabled=True)

            if event == "UNDO_BUTTON":
                for bttn in self.hidden_buttons:
                    self.window[bttn].update(visible=True)

                self.window["UNDO_BUTTON"].update(visible=False)
                self.window["GUESS_FRAME"].update(visible=False)
                self.window["GUESS_BUTTON"].update(visible=False)
                self.window[f"WBUTTON_{self.active_guess}"].update(disabled=False)
                self.hidden_buttons = []
                self.active_guess = None

            if event == "GUESS_BUTTON":
                print(vals)
                guess_bttn_tag = f"WBUTTON_{self.active_guess}"

                # Make the "guess frame" invisible
                self.window["GUESS_FRAME"].update(visible=False)

                # Use the "guess" function of the PypBoy class to return a list of possibilities
                new_list = self.guess(self.active_guess, vals['NUM_CORRECT'])

                # Dissappear the "Guess" and "Undo" buttons
                self.window["GUESS_BUTTON"].update(visible=False)
                self.window["UNDO_BUTTON"].update(visible=False)

                # Make the guessed word-button invisible since it's already been guessed
                self.window[guess_bttn_tag].update(visible=False)

                # If the length of the list is exactly or less than 1, inform the user that the visible button is the
                # answer they're looking for.
                if len(new_list) >= 1:
                    self.window["INSTRUCTION_TEXT"].update("Success! The button available below is your password!")

                # Iterate over the returned possibilities, making each associated button visible to be guessed next.
                for word in new_list:
                    bttn_tag = f"WBUTTON_{word}"
                    self.window[bttn_tag].update(visible=True, disabled=True)



import random_words

from pyp_boy.common.tools.errors import UncalledArgumentError


def generate_wordlist(length: int = 4,
                      count: int = 2,
                      debug: bool = False,
                      do_sort: bool = True,
                      no_dupes: bool = True,
                      catch_rejected: bool = False):
    """
        
            This function is included mainly for testing without having a puzzle to solve in front of you. There's no 
        guarantee that the items returned will share any letter-places. 
    |  
    
        Return Just a List:
    *******************
    Below is an example of running 'generate_wordlist' with no parameters. (Just using the defaults, hard-coded in)
    
    >>> wl = generate_wordlist()
    ['HUMP', 'PENS']
    
    Below sits yet another example, except this time we're specifying that we want 12 words in our list, each with a
    length of 5 letters.
    
    >>>  wl = generate_wordlist(length=5, count=12)
    ['DOSES', 'EASES', 'GRADE', 'GRAMS', 'HOPES', 'NOSES', 'RAISE', 'SAILS', 'SLAPS', 'SNOWS', 'SPANS', 'WELDS']
    
    
    As shown next, we can even get a bit granular on how that list is returned to us. We can ask it to not bother sorting the list before returning it, like so:
    
    >>> wl = generate_wordlist(length=4, count=20, do_sort=False)
    ['JUGS', 'HELM', 'SLED', 'STEP', 'HUGS', 'BOYS', 'DRIP', 'PANS', 'BOWS', 'FORK', 'MUCH', 'SAIL', 'OHMS', 'ARMY',
    'STOP', 'KITS', 'SALT', 'BUYS', 'DOCK', 'DRIP']
    
    
    >>> wl = generate_wordlist(length=6, count=15, debug=True)
    (['ROUNDS', 'PASSES', 'SWITCH', 'HOISTS', 'CREEKS', 'VIDEOS', 'GROUND', 'FARADS', 'ARREST', 'ROWERS', 'STEAMS',
    'BATTLE', 'SCOPES', 'DAMAGE', 'SAFETY'], 105)
        
        
        :param catch_rejected: 
        :param debug: **[ (bool) | Default: bool(False) ]**
        |  Run in debug mode? If you pass 'bool(True)' to this keyword argument the function will not only return your
        new wordlist but will return a tuple with the first element containing your wordlist, and the second element containing an integer representing the number of words fetched all-together (including words that were not equal to the length parameter).
    |  
        
        :param length: **[(int) | Default: int(4) ]** 
        | The integer representing the length of each the words contained within the list to be returned.
        
    |  
    
        :param count: **[ (int) | Default: int(2) ]**
        |  The integer representing the number of words the list should contain.
    |  
    
        :param do_sort: **[ (bool) | Default: bool(True) ]**
        |  If True (which is the default), the wordlist will be sorted alphabetically before being returned
    | 
    
        :param no_dupes: **[ (bool) | Default: bool(True) ]**
        |  *If True (which is the default)* | The function will check the list to see if a word already exists within the list, not adding it if this is the case.
        |  
        |  *If False* | The function will not process a check for duplicates before adding the word to the list
    
    |   
    """
    # Set some variables to be filled later

    # Create a var to keep track of how many words we grab to get our list (including words that didn't match the 
    # length indicated) if the 'debug' parameter is True 
    if debug:
        fetched_wcount: int = 0
        fetched_dupes: int = 0
        dupe_dict: dict = {}
        if catch_rejected:
            rej_wl = []
    else:
        if catch_rejected:
            raise UncalledArgumentError('catch-rejected', 'debug')

    w_list = []

    # Instantiate the RandomWords class
    rw = random_words.RandomWords()

    # Have the RandomWords package give us a random word, if it matches the length (in letters) that was asked for
    # upon calling the word will be added to the word list. This loops until 'w_list' has as many entries as
    # requested when called (using the parameter; 'count'
    while len(w_list) < count:

        if debug:
            fetched_wcount += 1

        word = rw.random_word()

        if len(word) == length:

            # If a dupe was found let's check to see if 'debug mode' is active. If so; we'll increment the
            # dupe count by one
            if word in w_list:
                if debug:
                    fetched_dupes += 1
                if not no_dupes:
                    w_list.append(word)
                else:
                    continue
            else:
                w_list.append(word)

        else:
            if catch_rejected and debug:
                rej_wl.append(word)

    if do_sort:
        w_list.sort()

    w_list = prepare_wordlist(w_list)

    if debug:
        stats = {
            "iterations": fetched_wcount,
            "duplicates": fetched_dupes,
            "rejected_words": ''
        }
        if catch_rejected:
            stats['rejected_words'] = rej_wl
        else:
            stats['rejected_words'] = 'Not caught'
        return w_list, stats
    else:
        return w_list


def prepare_wordlist(wordlist: list):
    """

    Calling this method with a word list as the "wordlist" parameter will return a word-list which is properly
    formatted (all caps, no whitespace characters. This method also makes sure that all words contained within the
    list are all the same length.

    Args:
        wordlist: A list containing two or more words to sanitize for GUI handling

    Returns:
        list: A word-list where all the words are all caps and arbitrary spaces removed.

    """

    # Since we might get a tuple during debugging, prepare to pull the actual word-list we want out of index 0
    if type(wordlist) == tuple:
        wordlist = wordlist[0]

    # Create a new list to be populated for return.
    new_list = []

    # Grab the length of the first word in the list to ensure uniformity with the other words.
    # TODO:
    #    * Put in checking of word length
    #    * Create an exception to be raised upon finding a word of non-conforming length
    #        * Rescue said exception smoothly, alert user of the problem and allow them to change the misfit entry
    word_length = len(wordlist[0])

    # Iterate through the original wordlist and do the following:
    #     1) Remove all whitespace from each word.
    #     2) Change the case of all letters in the words to 'upper'
    #     3) Add the newly-formatted word to the 'new_list' variable to be returned when the iterations are complete
    print(wordlist)
    for w in wordlist:
        w = w.replace(' ', '')
        w = w.upper()
        new_list.append(w)

    # Return list of newly formatted string entries, ready to be used by the PypBoy GUI!
    return new_list



class UncalledArgumentError(Exception):
    def __init__(self, dependant_arg, needed_arg):
        if dependant_arg == "catch-rejected":
            self.message = "You can't catch rejected words without also being in debug mode!"

        self.message += f"\nTarget Arg: {dependant_arg} = True"
        self.message += f"\nNeeded Arg: {needed_arg} = True"
        print(self.message)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pyp_boy', 'pyp_boy.common.tools', 'pyp_boy.gui']

package_data = \
{'': ['*']}

install_requires = \
['PySimpleGUI>=4.32.1,<5.0.0',
 'RandomWords>=0.3.0,<0.4.0',
 'inspy-logger==2.0-a3',
 'inspyre-toolbox>=1.0a2,<2.0']

entry_points = \
{'console_scripts': ['pyp-boy = pyp_boy.main:run_gui']}

setup_kwargs = {
    'name': 'pyp-boy',
    'version': '1.0a3',
    'description': 'Python-based program to assist with Fallout hacking. With a GUI!',
    'long_description': None,
    'author': 'Taylor-Jayde Blackstone',
    'author_email': 't.blackstone@inspyre.tech',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



def run_gui():
    from pyp_boy.gui import PypBoyGUI

    pbg = PypBoyGUI()
    main_win = pbg.MainWindow('Test Window')
    main_win.run_window()


if __name__ == 'pyp_boy.main' or __name__ == "__main__":
    run_gui()

else:
    print("PypBoy imported! Please bear in mind that this package was made to be used as a whole application.\n"
          "Importing this API is not recommended by the developer and therefore she can't promise that support will be "
          "available for problems with importing the API!! ")
    print(__name__)
